@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package Getopt::Long::Descriptive::Usage;
 # ABSTRACT: the usage description for GLD
-$Getopt::Long::Descriptive::Usage::VERSION = '0.099';
+$Getopt::Long::Descriptive::Usage::VERSION = '0.104';
 use List::Util qw(max);
 
 #pod =head1 SYNOPSIS
@@ -37,7 +37,7 @@ use List::Util qw(max);
 sub new {
   my ($class, $arg) = @_;
 
-  my @to_copy = qw(options leader_text show_defaults);
+  my @to_copy = qw(leader_text options show_defaults);
 
   my %copy;
   @copy{ @to_copy } = @$arg{ @to_copy };
@@ -86,14 +86,23 @@ sub option_text {
     my $desc = $opt->{desc};
     my $assign;
     if ($desc eq 'spacer') {
-      $string .= sprintf "$spec_fmt\n", $opt->{spec};
-      next;
+      if (ref $opt->{spec}) {
+        $string .= "${ $opt->{spec} }\n";
+        next;
+      } else {
+        my @lines = $self->_split_description($length, $opt->{spec});
+
+        $string .= length($_) ? sprintf("$spec_fmt\n", $_) : "\n" for @lines;
+        next;
+      }
     }
 
     ($spec, $assign) = Getopt::Long::Descriptive->_strip_assignment($spec);
-    $assign = _parse_assignment($assign);
-    $spec = join " ", reverse map { length > 1 ? "--${_}$assign" : "-${_}$assign" }
-                              split /\|/, $spec;
+    my ($left, $right) = _parse_assignment($assign);
+    $spec = join q{ },
+              reverse
+              map { length > 1 ? "--$left$_$right" : "-${_}$right" }
+              split /\|/, $spec;
 
     my @desc = $self->_split_description($length, $desc);
 
@@ -124,7 +133,9 @@ sub _option_length {
     my $number_shortopts = 0;
     my ($spec, $argspec) = Getopt::Long::Descriptive->_strip_assignment($fullspec);
     my $length = length $spec;
-    my $arglen = length(_parse_assignment($argspec));
+
+    my ($left, $right) = _parse_assignment($argspec);
+    my $arglen = length($left) + length($right);
 
     # Spacing rules:
     #
@@ -146,21 +157,19 @@ sub _option_length {
     }
 
     # Was the last option a "short" one?
-    if ($length - $last_pos == 2) {
+    if ($length - $last_pos == 1) {
         $number_shortopts++;
     }
 
     # We got $number_opts options, each with an argument length of
     # $arglen.  Plus each option (after the first) needs 3 a char
     # spacing.  $length gives us the total length of all options and 1
-    # char spacing per option (after the first).  It does not account
-    # for argument length and we want (at least) one additional char
-    # for space before the description.  So the result should be:
+    # char spacing per option (after the first).  So the result should be:
 
     my $number_longopts = $number_opts - $number_shortopts;
     my $total_arglen = $number_opts * $arglen;
     my $total_optsep = 2 * $number_longopts + $number_shortopts;
-    my $total = $length + $total_optsep + $total_arglen + 1;
+    my $total = $length + $total_optsep + $total_arglen;
     return $total;
 }
 
@@ -186,20 +195,24 @@ sub _split_description {
 
 sub _parse_assignment {
     my ($assign_spec) = @_;
-    my $argument;
+
     my $result = 'STR';
     my $desttype;
     if (length($assign_spec) < 2) {
         # empty, ! or +
-        return '';
+        return ('[no-]', '') if $assign_spec eq '!';
+        return ('', '');
     }
 
-    $argument = substr $assign_spec, 1, 2;
-    if ($argument eq 'i' or $argument eq 'o') {
+    my $optional = substr($assign_spec, 0, 1) eq ':';
+    my $argument = substr $assign_spec, 1, 2;
+
+    if ($argument =~ m/^[io]/ or $assign_spec =~ m/^:[+0-9]/) {
         $result = 'INT';
-    } elsif ($argument eq 'f') {
+    } elsif ($argument =~ m/^f/) {
         $result = 'NUM';
     }
+
     if (length($assign_spec) > 2) {
         $desttype = substr($assign_spec, 2, 1);
         if ($desttype eq '@') {
@@ -209,11 +222,13 @@ sub _parse_assignment {
             $result = "KEY=${result}...";
         }
     }
-    if (substr($assign_spec, 0, 1) eq ':') {
-        return "[=$result]";
+
+    if ($optional) {
+        return ("", "[=$result]");
     }
+
     # with leading space so it can just blindly be appended.
-    return " $result";
+    return ("", " $result");
 }
 
 #pod =head2 warn
@@ -230,7 +245,7 @@ sub warn { warn shift->text }
 #pod
 #pod   $usage_obj->die(\%arg);
 #pod
-#pod Some arguments can be provided 
+#pod Some arguments can be provided
 #pod
 #pod   pre_text  - text to be prepended to the usage message
 #pod   post_text - text to be appended to the usage message
@@ -275,7 +290,7 @@ Getopt::Long::Descriptive::Usage - the usage description for GLD
 
 =head1 VERSION
 
-version 0.099
+version 0.104
 
 =head1 SYNOPSIS
 
@@ -326,7 +341,7 @@ This throws the usage message as an exception.
 
   $usage_obj->die(\%arg);
 
-Some arguments can be provided 
+Some arguments can be provided
 
   pre_text  - text to be prepended to the usage message
   post_text - text to be appended to the usage message
